@@ -23,12 +23,13 @@ from mpi4py import MPI
 comm = MPI.COMM_WORLD
 rank = comm.Get_rank()
 
-def do_Boltz_tensors ( data_controller, smearing, temp, ene, velkp, ispin ):
+def do_Boltz_tensors ( data_controller, smearing, temp, ene, velkp, ispin, channels, weights):
   # Compute the L_alpha tensors for Boltzmann transport
 
   arrays,attributes = data_controller.data_dicts()
 
   esize = ene.size
+  arrays['scattering_tau'] = get_tau(data_controller, temp, channels, weights)
 
 #### Forced t_tensor to have all components
   t_tensor = np.array([[0,0],[1,1],[2,2],[0,1],[0,2],[1,2]], dtype=int)
@@ -49,9 +50,6 @@ def do_Boltz_tensors ( data_controller, smearing, temp, ene, velkp, ispin ):
     sym = lambda L : (L[0,1], L[0,2], L[1,2])
     L0[1,0],L0[2,0],L0[2,1] = sym(L0)
 
-#  L1 = L2 = None
-#  if smearing is None:
-
   L1 = zoz(rank)
   L1aux = fLloop(1)
   comm.Reduce(L1aux, L1, op=MPI.SUM)
@@ -69,8 +67,11 @@ def do_Boltz_tensors ( data_controller, smearing, temp, ene, velkp, ispin ):
   return (L0, L1, L2) if rank==0 else (None, None, None)
 
 
-def get_tau ( data_controller, channels ):
+def get_tau ( data_controller, temp, channels, weights ):
   import numpy as np
+  import scipy.constants as cp
+  from .TauModel import TauModel
+  from .do_tau_models import builtin_tau_model
 
   arry,attr = data_controller.data_dicts()
   snktot = arry['E_k'].shape[0]
@@ -81,14 +82,42 @@ def get_tau ( data_controller, channels ):
       a_tau = np.ones((snktot), dtype=float)
       taus.append(a_tau)
 
-    if c == 'optical':
-      o_tau = np.ones((snktot), dtype=float)
-      taus.append(o_tau)
+  bnd = attr['bnd']
+  eigs = np.abs(arry['E_k'][:,:bnd,:])
+  snktot,_,nspin = eigs.shape
+
+  models = []
+  if channels != None:
+    if len(weights) == 0:
+      weights = np.ones(len(channels))
+    elif len(weights) != len(models):
+      raise Exception('Length of weights does not match the number of channels.')
+    for i,c in enumerate(channels):
+      if isinstance(c,str):
+        models.append(builtin_tau_model(c,attr['tau_dict'],weights[i]))
+      elif isinstance(c,TauModel):
+        c.weight = weights[i]
+        models.append(c)
+      else:
+        print('Invalid channel type.')
+
+  if len(models) == 0:
+    # Constant relaxation time approximation with tau = 1
+    tau = np.ones((snktot,bnd,nspin), dtype=float)
+
+  else:
+    # Compute tau as a harmonic sum of scattering channel contributions.
+    tau = np.zeros((snktot,bnd,nspin), dtype=float)
+    for m in models:
+      try:
+        tau += m.weight/m.evaluate(temp, eigs)
+      except KeyError as e:
+        print('Ensure that all required parameters are specified in the provided dictionary.')
+        print(e)
+    tau = 1/tau
+
+  return tau
 
-  tau = np.zeros((snktot), dtype=float)
-  for t in taus:
-    tau += 1./t
-  return len(channels)/tau
 
 
 def L_loop ( data_controller, temp, smearing, ene, velkp, t_tensor, alpha, ispin ):
@@ -115,94 +144,24 @@ def L_loop ( data_controller, temp, smearing, ene, velkp, t_tensor, alpha, ispin
     EtoAlpha = np.power(Eaux[:,:]-ene, alpha)
     if smearing is None:
       Eaux -= ene
-#      smearA = .5/(temp*(1.+.5*(np.exp(Eaux/temp)+np.exp(-Eaux/temp))))
       smearA = 1/(4*temp*(np.cosh(Eaux/(2*temp))**2))
     else:
       if smearing == 'gauss':
-#        if rank == 0 and n == 0: print('new smearing')
-#        smearA = np.exp(-((ene-Eaux)/delk)**2)/temp)/np.sqrt(np.pi)
         smearA = gaussian(Eaux, ene, delk)
-#        smearA = gaussian(Eaux, ene, delk)
       elif smearing == 'm-p':
         smearA = metpax(Eaux, ene, delk)
     for l in range(t_tensor.shape[0]):
       i = t_tensor[l][0]
       j = t_tensor[l][1]
-      tau = get_tau(data_controller, ['acoustic', 'optical'])
-      L[i,j,:] += np.sum(kq_wght*tau*velkp[:,i,n,ispin]*velkp[:,j,n,ispin]*(smearA*EtoAlpha).T, axis=1)
+      L[i,j,:] += np.sum(kq_wght*arrays['scattering_tau'][:,n,ispin]*velkp[:,i,n,ispin]*velkp[:,j,n,ispin]*(smearA*EtoAlpha).T, axis=1)
+  '''
   # noise reduction using a running average (correlation function)
+  # Only possible for sigma vs chemical potential
   win = int(esize*0.025)
   N = esize
   for l in range(t_tensor.shape[0]):
     i = t_tensor[l][0]
     j = t_tensor[l][1]
     L[i,j,:] = signal.correlate(L[i,j,:] , np.ones(win), mode='same', method='fft')/win
-  
+  '''
   return L
-
-
-def do_Hall_tensors( E_k_range,velkp_range,d2Ed2k_range,kq_wght,temp,ene ):
-  # calculated components of the Hall tensor R_ijk
-
-
-  if rank==0:
-      R = np.zeros((3,3,3,ene.size),dtype=float,order="C")
-  else: R = None
-
-  Raux = H_loop(ene,E_k_range,velkp_range,d2Ed2k_range,kq_wght,temp)
-  comm.Reduce(Raux,R,op=MPI.SUM)
-
-  return(R)
-
-
-
-def H_loop(ene,E_k,velkp,d2Ed2k,kq_wght,temp):
-  # We assume tau=1 in the constant relaxation time approximation
-
-  R = np.zeros((3,3,3,ene.size),dtype=float,order="C")
-
-  # mapping of the unique 2nd rank tensor components
-  d2Ed2k_ind = np.array([[0,3,5],
-                       [3,1,4],
-                       [5,4,2]],dtype=int,order="C")
-
-  # precompute the 6 unique v_i*v_j
-  v2=np.zeros((6,E_k.shape[0]),dtype=float,order="C")
-  ij_ind = np.array([[0,0],[1,1],[2,2],[0,1],[1,2],[0,2]],dtype=int)
-
-  for l in range(ij_ind.shape[0]):
-      i     = ij_ind[l][0]
-      j     = ij_ind[l][1]
-      v2[l] = velkp[i]*velkp[j]
-
-  # precompute sig_xyz
-  sig_xyz=np.zeros((3,3,3,E_k.shape[0]),order="C")    
-  for a in range(3):
-      for b in range(3):
-          if a==b and b==0: continue
-          sig_xyz[a,b,0] = d2Ed2k[d2Ed2k_ind[b,1]]*v2[d2Ed2k_ind[a,2]] - \
-                           d2Ed2k[d2Ed2k_ind[b,2]]*v2[d2Ed2k_ind[a,1]]
-  for a in range(3):
-      for b in range(3):                          
-          if a==b and b==1: continue
-          sig_xyz[a,b,1] = d2Ed2k[d2Ed2k_ind[b,2]]*v2[d2Ed2k_ind[a,0]] - \
-                           d2Ed2k[d2Ed2k_ind[b,0]]*v2[d2Ed2k_ind[a,2]]
-  for a in range(3):
-      for b in range(3):                          
-          if a==b and b==2: continue 
-          sig_xyz[a,b,2] = d2Ed2k[d2Ed2k_ind[b,0]]*v2[d2Ed2k_ind[a,1]] - \
-                           d2Ed2k[d2Ed2k_ind[b,1]]*v2[d2Ed2k_ind[a,0]]
-
-
-  for n in range(ene.shape[0]):        
-      # dfermi(e-e_f)/de
-      Eaux = 1.0/(1.0+np.cosh((E_k-ene[n])/temp))
-      for a in range(3):
-          for b in range(3):
-              for g in range(3):                    
-                  if a==b and b==g: continue                                           
-                  R[a,b,g,n] = np.sum(sig_xyz[a,b,g]*Eaux)
-
-  return(R*kq_wght[0]*0.5/temp)
-
-
